/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.navigator;

import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IDecoratorManager;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.internal.NavigatorService;
import org.fortiss.tooling.kernel.services.IActionService;
import org.fortiss.tooling.kernel.services.IContextMenuService;
import org.fortiss.tooling.kernel.services.IEditorService;
import org.fortiss.tooling.kernel.services.INavigatorService;
import org.fortiss.tooling.kernel.services.IPropertiesService;
import org.fortiss.tooling.kernel.util.EObjectSelectionUtils;
import org.unicase.ecp.model.ECPWorkspaceManager;
import org.unicase.ecp.model.NoWorkspaceException;
import org.unicase.ecp.model.workSpaceModel.ECPProject;
import org.unicase.ecp.model.workSpaceModel.ECPProjectListener;
import org.unicase.ecp.model.workSpaceModel.ECPWorkspace;
import org.unicase.ecp.model.workSpaceModel.WorkSpaceModelPackage;

/**
 * {@link ViewPart} of the model navigator provided by the tooling kernel.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public final class NavigatorViewPart extends ViewPart implements
		ISelectionListener, IDoubleClickListener,
		ITabbedPropertySheetPageContributor {

	/** Stores the TreeViewer. */
	private TreeViewer viewer;

	/** Stores the EmfStore workspace. */
	private ECPWorkspace workspace;

	/** Stores the workspace listener. */
	private AdapterImpl workspaceListener;

	/** Stores the project listener. */
	private ECPProjectListener projectListener;

	/** Stores the menu manager. */
	private MenuManager menuManager;

	/** Stores the UI update job. */
	private final UIJob updateUI = new UIJob("Update Model Navigator") {
		@Override
		public IStatus runInUIThread(IProgressMonitor monitor) {
			viewer.refresh();
			return Status.OK_STATUS;
		}
	};

	/** Constructor. */
	public NavigatorViewPart() {
		try {
			workspace = ECPWorkspaceManager.getInstance().getWorkSpace();
		} catch (NoWorkspaceException e) {
			LoggingUtils.log(ToolingKernelActivator.getDefault(),
					e.getMessage(), IStatus.ERROR, e);
			return;
		}

		createProjectListener();
		for (ECPProject project : workspace.getProjects()) {
			project.addECPProjectListener(projectListener);
		}

		createWorkspaceListener();
		workspace.eAdapters().add(workspaceListener);

		((NavigatorService) INavigatorService.INSTANCE)
				.setNavigatorViewPart(this);
	}

	/**
	 * Creates the workspace listener, which registers the project listener on
	 * added projects. It also unregisters the project listener from removed
	 * projects.
	 */
	private void createWorkspaceListener() {
		workspaceListener = new AdapterImpl() {

			@Override
			public void notifyChanged(Notification msg) {
				if ((msg.getFeatureID(ECPWorkspace.class)) == WorkSpaceModelPackage.ECP_WORKSPACE__PROJECTS) {
					if (msg.getEventType() == Notification.ADD
							&& WorkSpaceModelPackage.eINSTANCE.getECPProject()
									.isInstance(msg.getNewValue())) {
						ECPProject projectSpace = (ECPProject) msg
								.getNewValue();
						projectSpace.addECPProjectListener(projectListener);
					} else if (msg.getEventType() == Notification.REMOVE
							&& WorkSpaceModelPackage.eINSTANCE.getECPProject()
									.isInstance(msg.getOldValue())) {
						ECPProject projectSpace = (ECPProject) msg
								.getOldValue();
						projectSpace.removeECPProjectListener(projectListener);
					}
					updateUI.schedule();
				}
				super.notifyChanged(msg);
			}
		};
	}

	/**
	 * Creates the project listener, which refreshes the viewer upon any
	 * changes.
	 */
	private void createProjectListener() {
		projectListener = new ECPProjectListener() {
			@Override
			public void projectDeleted() {
				// ignore
			}

			@Override
			public void projectChanged() {
				updateUI.schedule();
			}

			@Override
			public void modelelementDeleted(EObject eobject) {
				// ignore
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI);

		if (workspace != null) {
			// NOTE that the order of this important. See also JDT package
			// explorer.
			IDecoratorManager decoratorManager = PlatformUI.getWorkbench()
					.getDecoratorManager();
			viewer.setLabelProvider(new DecoratingLabelProvider(
					new NavigatorTreeLabelProvider(), decoratorManager
							.getLabelDecorator()));
			viewer.setContentProvider(new NavigatorTreeContentProvider());

			createContextMenu();
			getSite().setSelectionProvider(viewer);

			viewer.setInput(workspace);

			viewer.addDoubleClickListener(this);

			IActionService.INSTANCE.registerGlobalActions(getViewSite());

			PlatformUI.getWorkbench().getActiveWorkbenchWindow()
					.getSelectionService().addSelectionListener(this);
		}

		// FIXME: whats that good for?
		// if (viewer.getTree().getItems().length > 0) {
		// setActiveECPProject(viewer.getTree().getItem(0).getData());
		// viewer.getTree().select(viewer.getTree().getItem(0));
		// }
	}

	/** Creates the context menu. */
	private void createContextMenu() {
		menuManager = IContextMenuService.INSTANCE
				.createDefaultContextMenu(null);

		Menu contextMenu = menuManager.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(contextMenu);

		getSite().registerContextMenu(menuManager, viewer);
	}

	/** Creates the actions. */

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		EObject selectedElement = EObjectSelectionUtils
				.getCurrentSelectionFirstElement();
		if (selectedElement != null) {
			IActionService.INSTANCE.update();
		}
		// TODO (FH): implement link with editor feature here
	}

	/** Sets the active ECPProject element. */
	private void setActiveECPProject(Object obj) {
		if (obj instanceof EObject) {
			try {
				ECPWorkspaceManager.getInstance().getWorkSpace()
						.setActiveModelelement((EObject) obj);
			} catch (NoWorkspaceException e) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						e.getMessage(), e);
			}
		}

	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		((NavigatorService) INavigatorService.INSTANCE)
				.setNavigatorViewPart(null);

		PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getSelectionService().removeSelectionListener(this);
		getSite().setSelectionProvider(null);

		workspace.eAdapters().remove(workspaceListener);
		for (ECPProject project : workspace.getProjects()) {
			project.removeECPProjectListener(projectListener);
		}

		super.dispose();
	}

	/** {@inheritDoc} */
	@Override
	public void doubleClick(DoubleClickEvent event) {
		if (event.getSelection() instanceof IStructuredSelection) {
			EObject element = EObjectSelectionUtils.getFirstElement(event
					.getSelection());
			if (element != null) {
				IEditorService.INSTANCE.openInEditor(element);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public String getContributorId() {
		return IPropertiesService.TABBED_PROPERTY_CONTRIBUTOR_ID;
	}

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySheetPage.class) {
			return new TabbedPropertySheetPage(this);
		}
		return super.getAdapter(adapter);
	}
}
